// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/aiplatform/v1/llm_utility_service.proto

#include "google/cloud/aiplatform/v1/internal/llm_utility_connection_impl.h"
#include "google/cloud/aiplatform/v1/internal/llm_utility_option_defaults.h"
#include "google/cloud/background_threads.h"
#include "google/cloud/common_options.h"
#include "google/cloud/grpc_options.h"
#include "google/cloud/internal/retry_loop.h"
#include <memory>

namespace google {
namespace cloud {
namespace aiplatform_v1_internal {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
namespace {

std::unique_ptr<aiplatform_v1::LlmUtilityServiceRetryPolicy> retry_policy(
    Options const& options) {
  return options.get<aiplatform_v1::LlmUtilityServiceRetryPolicyOption>()
      ->clone();
}

std::unique_ptr<BackoffPolicy> backoff_policy(Options const& options) {
  return options.get<aiplatform_v1::LlmUtilityServiceBackoffPolicyOption>()
      ->clone();
}

std::unique_ptr<aiplatform_v1::LlmUtilityServiceConnectionIdempotencyPolicy>
idempotency_policy(Options const& options) {
  return options
      .get<aiplatform_v1::LlmUtilityServiceConnectionIdempotencyPolicyOption>()
      ->clone();
}

}  // namespace

LlmUtilityServiceConnectionImpl::LlmUtilityServiceConnectionImpl(
    std::unique_ptr<google::cloud::BackgroundThreads> background,
    std::shared_ptr<aiplatform_v1_internal::LlmUtilityServiceStub> stub,
    Options options)
    : background_(std::move(background)),
      stub_(std::move(stub)),
      options_(internal::MergeOptions(
          std::move(options), LlmUtilityServiceConnection::options())) {}

StatusOr<google::cloud::aiplatform::v1::CountTokensResponse>
LlmUtilityServiceConnectionImpl::CountTokens(
    google::cloud::aiplatform::v1::CountTokensRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->CountTokens(request),
      [this](grpc::ClientContext& context,
             google::cloud::aiplatform::v1::CountTokensRequest const& request) {
        return stub_->CountTokens(context, request);
      },
      request, __func__);
}

StatusOr<google::cloud::aiplatform::v1::ComputeTokensResponse>
LlmUtilityServiceConnectionImpl::ComputeTokens(
    google::cloud::aiplatform::v1::ComputeTokensRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->ComputeTokens(request),
      [this](
          grpc::ClientContext& context,
          google::cloud::aiplatform::v1::ComputeTokensRequest const& request) {
        return stub_->ComputeTokens(context, request);
      },
      request, __func__);
}

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace aiplatform_v1_internal
}  // namespace cloud
}  // namespace google
